﻿Imports System.Windows.Media.Animation

''' <summary>
''' Extends the C1RadialGauge to provide an animation when the Value
''' shown on the gauge changes.
''' </summary>
Public Class C1AnimatedGauge
    Inherits C1.WPF.Gauge.C1RadialGauge
    ''' <summary>
    ''' Gets or sets the target for the control's Value property.
    ''' </summary>
    Public Property AnimatedValue() As Double
        Get
            Return CDbl(GetValue(AnimatedValueProperty))
        End Get
        Set(value As Double)
            SetValue(AnimatedValueProperty, value)
        End Set
    End Property
    ''' <summary>
    ''' Identifies the <see cref="AnimatedValue"/> dependency property.
    ''' </summary>
    Public Shared ReadOnly AnimatedValueProperty As DependencyProperty = DependencyProperty.Register("AnimatedValue", GetType(Double), GetType(C1AnimatedGauge), New PropertyMetadata(AddressOf OnTargetValuePropertyChanged))
    Private Shared Sub OnTargetValuePropertyChanged(d As DependencyObject, e As DependencyPropertyChangedEventArgs)
        ' get animated gauge
        Dim ag = DirectCast(d, C1AnimatedGauge)

        ' create animation
        Dim da = New DoubleAnimation()
        da.[To] = CDbl(e.NewValue)
        da.Duration = New Duration(TimeSpan.FromMilliseconds(ag.Duration))
        Storyboard.SetTargetProperty(da, New PropertyPath("Value"))
        Storyboard.SetTarget(da, d)

        ' apply easing function
        Dim ef = ag.EasingFunction
        If ef Is Nothing Then
            Dim ee = New ElasticEase()
            ee.Oscillations = 1
            ee.Springiness = 3
            ef = ee
        End If
        da.EasingFunction = ef

        ' play animation
        Dim sb = New Storyboard()
        sb.Children.Add(da)
        sb.Begin()
    End Sub
    ''' <summary>
    ''' Gets or sets the easing function that animates the gauge pointer.
    ''' </summary>
    Public Property EasingFunction() As IEasingFunction
        Get
            Return DirectCast(GetValue(EasingFunctionProperty), IEasingFunction)
        End Get
        Set(value As IEasingFunction)
            SetValue(EasingFunctionProperty, value)
        End Set
    End Property
    ''' <summary>
    ''' Identifies the <see cref="EasingFunction"/> dependency property.
    ''' </summary>
    Public Shared ReadOnly EasingFunctionProperty As DependencyProperty = DependencyProperty.Register("EasingFunction", GetType(IEasingFunction), GetType(C1AnimatedGauge), New PropertyMetadata(Nothing))
    ''' <summary>
    ''' Gets or sets the duration of the animation, in milliseconds.
    ''' </summary>
    Public Property Duration() As Double
        Get
            Return CDbl(GetValue(DurationProperty))
        End Get
        Set(value As Double)
            SetValue(DurationProperty, value)
        End Set
    End Property
    ''' <summary>
    ''' Identifies the <see cref="Duration"/> dependency property.
    ''' </summary>
    Public Shared ReadOnly DurationProperty As DependencyProperty = DependencyProperty.Register("Duration", GetType(Double), GetType(C1AnimatedGauge), New PropertyMetadata(250.0))
End Class

